@@ -1,57 +1,64 @@
+import difflib
 import email.parser
+import inspect
 import json
 import os
 import re
+import sys
+
 import pytest
 
-from h2_conf import HttpdConf
+from .env import H2Conf
 
 
-class TestStore:
+class TestPost:
 
     @pytest.fixture(autouse=True, scope='class')
     def _class_scope(self, env):
-        env.setup_data_1k_1m()
-        HttpdConf(env).add_vhost_cgi().install()
+        TestPost._local_dir = os.path.dirname(inspect.getfile(TestPost))
+        H2Conf(env).add_vhost_cgi().install()
         assert env.apache_restart() == 0
 
+    def local_src(self, fname):
+        return os.path.join(TestPost._local_dir, fname)
+
     # upload and GET again using curl, compare to original content
     def curl_upload_and_verify(self, env, fname, options=None):
         url = env.mkurl("https", "cgi", "/upload.py")
         fpath = os.path.join(env.gen_dir, fname)
         r = env.curl_upload(url, fpath, options=options)
-        assert r.exit_code == 0, r.stderr
-        assert r.response["status"] >= 200 and r.response["status"] < 300
+        assert r.exit_code == 0, f"{r}"
+        assert 200 <= r.response["status"] < 300
 
         r2 = env.curl_get(r.response["header"]["location"])
         assert r2.exit_code == 0
         assert r2.response["status"] == 200
-        with open(env.test_src(fpath), mode='rb') as file:
+        with open(self.local_src(fpath), mode='rb') as file:
             src = file.read()
         assert src == r2.response["body"]
 
-    def test_004_01(self, env):
-        self.curl_upload_and_verify(env, "data-1k", ["--http1.1"])
+    def test_h2_004_01(self, env):
+        self.curl_upload_and_verify(env, "data-1k", ["-vvv", "--http1.1"])
         self.curl_upload_and_verify(env, "data-1k", ["--http2"])
 
-    def test_004_02(self, env):
+    def test_h2_004_02(self, env):
         self.curl_upload_and_verify(env, "data-10k", ["--http1.1"])
         self.curl_upload_and_verify(env, "data-10k", ["--http2"])
 
-    def test_004_03(self, env):
+    def test_h2_004_03(self, env):
         self.curl_upload_and_verify(env, "data-100k", ["--http1.1"])
         self.curl_upload_and_verify(env, "data-100k", ["--http2"])
 
-    def test_004_04(self, env):
+    def test_h2_004_04(self, env):
         self.curl_upload_and_verify(env, "data-1m", ["--http1.1"])
         self.curl_upload_and_verify(env, "data-1m", ["--http2"])
 
-    def test_004_05(self, env):
+    def test_h2_004_05(self, env):
         self.curl_upload_and_verify(env, "data-1k", ["-v", "--http1.1", "-H", "Expect: 100-continue"])
         self.curl_upload_and_verify(env, "data-1k", ["-v", "--http2", "-H", "Expect: 100-continue"])
 
     @pytest.mark.skipif(True, reason="python3 regresses in chunked inputs to cgi")
-    def test_004_06(self, env):
+    def test_h2_004_06(self, env):
         self.curl_upload_and_verify(env, "data-1k", ["--http1.1", "-H", "Content-Length: "])
         self.curl_upload_and_verify(env, "data-1k", ["--http2", "-H", "Content-Length: "])
 
@@ -63,7 +70,7 @@ class TestStore:
         ("H2_STREAM_ID", "1"),
         ("H2_STREAM_TAG", r'\d+-1'),
     ])
-    def test_004_07(self, env, name, value):
+    def test_h2_004_07(self, env, name, value):
         url = env.mkurl("https", "cgi", "/env.py")
         r = env.curl_post_value(url, "name", name)
         assert r.exit_code == 0
@@ -72,26 +79,6 @@ class TestStore:
         assert m
         assert re.match(value, m.group(1)) 
 
-    # verify that we parse nghttp output correctly
-    def check_nghttp_body(self, env, ref_input, nghttp_output):
-        with open(env.test_src(os.path.join(env.gen_dir, ref_input)), mode='rb') as f:
-            refbody = f.read()
-        with open(env.test_src(nghttp_output), mode='rb') as f:
-            text = f.read()
-        o = env.nghttp().parse_output(text)
-        assert "response" in o
-        assert "body" in o["response"]
-        if refbody != o["response"]["body"]:
-            with open(env.test_src(os.path.join(env.gen_dir, '%s.parsed' % ref_input)), mode='bw') as f:
-                f.write(o["response"]["body"])
-        assert len(refbody) == len(o["response"]["body"])
-        assert refbody == o["response"]["body"]
-    
-    def test_004_20(self, env):
-        self.check_nghttp_body(env, 'data-1k', 'data/nghttp-output-1k-1.txt')
-        self.check_nghttp_body(env, 'data-10k', 'data/nghttp-output-10k-1.txt')
-        self.check_nghttp_body(env, 'data-100k', 'data/nghttp-output-100k-1.txt')
-
     # POST some data using nghttp and see it echo'ed properly back
     def nghttp_post_and_verify(self, env, fname, options=None):
         url = env.mkurl("https", "cgi", "/echo.py")
@@ -101,20 +88,30 @@ class TestStore:
         assert r.exit_code == 0
         assert r.response["status"] >= 200 and r.response["status"] < 300
 
-        with open(env.test_src(fpath), mode='rb') as file:
+        with open(self.local_src(fpath), mode='rb') as file:
             src = file.read()
-        assert src == r.response["body"]
+        assert 'request-length' in r.response["header"]
+        assert int(r.response["header"]['request-length']) == len(src)
+        if len(r.response["body"]) != len(src):
+            sys.stderr.writelines(difflib.unified_diff(
+                src.decode().splitlines(True),
+                r.response["body"].decode().splitlines(True),
+                fromfile='source',
+                tofile='response'
+            ))
+            assert len(r.response["body"]) == len(src)
+        assert r.response["body"] == src, f"expected '{src}', got '{r.response['body']}'"
 
     @pytest.mark.parametrize("name", [
         "data-1k", "data-10k", "data-100k", "data-1m"
     ])
-    def test_004_21(self, env, name):
+    def test_h2_004_21(self, env, name):
         self.nghttp_post_and_verify(env, name, [])
 
     @pytest.mark.parametrize("name", [
-        "data-1k", "data-10k", "data-100k", "data-1m"
+        "data-1k", "data-10k", "data-100k", "data-1m",
     ])
-    def test_004_22(self, env, name, repeat):
+    def test_h2_004_22(self, env, name, repeat):
         self.nghttp_post_and_verify(env, name, ["--no-content-length"])
 
     # upload and GET again using nghttp, compare to original content
@@ -130,29 +127,29 @@ class TestStore:
         r2 = env.nghttp().get(r.response["header"]["location"])
         assert r2.exit_code == 0
         assert r2.response["status"] == 200
-        with open(env.test_src(fpath), mode='rb') as file:
+        with open(self.local_src(fpath), mode='rb') as file:
             src = file.read()
         assert src == r2.response["body"]
 
     @pytest.mark.parametrize("name", [
         "data-1k", "data-10k", "data-100k", "data-1m"
     ])
-    def test_004_23(self, env, name, repeat):
+    def test_h2_004_23(self, env, name, repeat):
         self.nghttp_upload_and_verify(env, name, [])
 
     @pytest.mark.parametrize("name", [
         "data-1k", "data-10k", "data-100k", "data-1m"
     ])
-    def test_004_24(self, env, name, repeat):
+    def test_h2_004_24(self, env, name, repeat):
         self.nghttp_upload_and_verify(env, name, ["--expect-continue"])
 
     @pytest.mark.parametrize("name", [
         "data-1k", "data-10k", "data-100k", "data-1m"
     ])
-    def test_004_25(self, env, name, repeat):
+    def test_h2_004_25(self, env, name, repeat):
         self.nghttp_upload_and_verify(env, name, ["--no-content-length"])
 
-    def test_004_30(self, env):
+    def test_h2_004_30(self, env):
         # issue: #203
         resource = "data-1k"
         full_length = 1000
@@ -161,18 +158,18 @@ class TestStore:
         logfile = os.path.join(env.server_logs_dir, "test_004_30")
         if os.path.isfile(logfile):
             os.remove(logfile)
-        HttpdConf(env).add("""
+        H2Conf(env).add("""
 LogFormat "{ \\"request\\": \\"%r\\", \\"status\\": %>s, \\"bytes_resp_B\\": %B, \\"bytes_tx_O\\": %O, \\"bytes_rx_I\\": %I, \\"bytes_rx_tx_S\\": %S }" issue_203
 CustomLog logs/test_004_30 issue_203
         """).add_vhost_cgi().install()
         assert env.apache_restart() == 0
         url = env.mkurl("https", "cgi", "/files/{0}".format(resource))
-        r = env.curl_get(url, 5, ["--http2"])
-        assert 200 == r.response["status"]
-        r = env.curl_get(url, 5, ["--http1.1", "-H", "Range: bytes=0-{0}".format(chunk-1)])
+        r = env.curl_get(url, 5, options=["--http2"])
+        assert r.response["status"] == 200
+        r = env.curl_get(url, 5, options=["--http1.1", "-H", "Range: bytes=0-{0}".format(chunk-1)])
         assert 206 == r.response["status"]
         assert chunk == len(r.response["body"].decode('utf-8'))
-        r = env.curl_get(url, 5, ["--http2", "-H", "Range: bytes=0-{0}".format(chunk-1)])
+        r = env.curl_get(url, 5, options=["--http2", "-H", "Range: bytes=0-{0}".format(chunk-1)])
         assert 206 == r.response["status"]
         assert chunk == len(r.response["body"].decode('utf-8'))
         # now check what response lengths have actually been reported
@@ -190,7 +187,7 @@ CustomLog logs/test_004_30 issue_203
         assert log_h2['bytes_resp_B'] == chunk
         assert log_h2['bytes_tx_O'] > chunk
         
-    def test_004_40(self, env):
+    def test_h2_004_40(self, env):
         # echo content using h2test_module "echo" handler
         def post_and_verify(fname, options=None):
             url = env.mkurl("https", "cgi", "/h2test/echo")
@@ -211,7 +208,7 @@ CustomLog logs/test_004_30 issue_203
                 if fname == part.get_filename():
                     filepart = part
             assert filepart
-            with open(env.test_src(fpath), mode='rb') as file:
+            with open(self.local_src(fpath), mode='rb') as file:
                 src = file.read()
             assert src == filepart.get_payload(decode=True)
         
