@@ -3,15 +3,15 @@ import time
 
 import pytest
 
-from h2_conf import HttpdConf
-from h2_curl import CurlPiper
+from .env import H2Conf
+from pyhttpd.curl import CurlPiper
 
 
-class TestStore:
+class TestTimeout:
 
     # Check that base servers 'Timeout' setting is observed on SSL handshake
-    def test_105_01(self, env):
-        conf = HttpdConf(env)
+    def test_h2_105_01(self, env):
+        conf = H2Conf(env)
         conf.add("""
             AcceptFilter http none
             Timeout 1.5
@@ -44,8 +44,8 @@ class TestStore:
         sock.close()
 
     # Check that mod_reqtimeout handshake setting takes effect
-    def test_105_02(self, env):
-        conf = HttpdConf(env)
+    def test_h2_105_02(self, env):
+        conf = H2Conf(env)
         conf.add("""
             AcceptFilter http none
             Timeout 10
@@ -80,8 +80,8 @@ class TestStore:
 
     # Check that mod_reqtimeout handshake setting do no longer apply to handshaked 
     # connections. See <https://github.com/icing/mod_h2/issues/196>.
-    def test_105_03(self, env):
-        conf = HttpdConf(env)
+    def test_h2_105_03(self, env):
+        conf = H2Conf(env)
         conf.add("""
             Timeout 10
             RequestReadTimeout handshake=1 header=5 body=10
@@ -90,17 +90,17 @@ class TestStore:
         conf.install()
         assert env.apache_restart() == 0
         url = env.mkurl("https", "cgi", "/necho.py")
-        r = env.curl_get(url, 5, [
+        r = env.curl_get(url, 5, options=[
             "-vvv",
             "-F", ("count=%d" % 100),
             "-F", ("text=%s" % "abcdefghijklmnopqrstuvwxyz"),
             "-F", ("wait1=%f" % 1.5),
         ])
-        assert 200 == r.response["status"]
+        assert r.response["status"] == 200
 
-    def test_105_10(self, env):
+    def test_h2_105_10(self, env):
         # just a check without delays if all is fine
-        conf = HttpdConf(env)
+        conf = H2Conf(env)
         conf.add_vhost_cgi()
         conf.install()
         assert env.apache_restart() == 0
@@ -111,11 +111,11 @@ class TestStore:
         assert piper.exitcode == 0
         assert len("".join(stdout)) == 3 * 8192
 
-    @pytest.mark.skipif(True, reason="new feature in upcoming http2")
-    def test_105_11(self, env):
+    @pytest.mark.skip(reason="only in 2.5.x")
+    def test_h2_105_11(self, env):
         # short connection timeout, longer stream delay
         # receiving the first response chunk, then timeout
-        conf = HttpdConf(env)
+        conf = H2Conf(env)
         conf.add_vhost_cgi()
         conf.add("Timeout 1")
         conf.install()
@@ -126,11 +126,11 @@ class TestStore:
         stdout, stderr = piper.close()
         assert len("".join(stdout)) == 8192
 
-    @pytest.mark.skipif(True, reason="new feature in upcoming http2")
-    def test_105_12(self, env):
+    @pytest.mark.skip(reason="only in 2.5.x")
+    def test_h2_105_12(self, env):
         # long connection timeout, short stream timeout
         # sending a slow POST
-        conf = HttpdConf(env)
+        conf = H2Conf(env)
         conf.add_vhost_cgi()
         conf.add("Timeout 10")
         conf.add("H2StreamTimeout 1")
